extern crate core;

mod helpers;
mod middlewares;
mod models;
mod routes;
mod schema;
mod services;
mod utils;

use crate::helpers::db;

#[actix_web::main]
async fn main() -> std::io::Result<()> {
    use actix_web::{App, HttpServer};

    dotenvy::dotenv().expect("No .env file found");
    env_logger::init();

    // Register services.
    let _ = services::spotify::instance().await;

    db::init();

    HttpServer::new(move || {
        App::new()
            .wrap(middlewares::cache::Caching)
            .service(routes::auth::routes())
            // .wrap(middlewares::auth::auth())
            .service(routes::playlists::routes())
            .service(routes::me::routes())
            .service(routes::import::routes())
            .service(routes::users::routes())
            .service(routes::search::routes())
    })
    .bind(("127.0.0.1", 9000))?
    .run()
    .await
}
