use crate::helpers::db;
use crate::schema::{artists, artists_tracks};
use chrono::NaiveDateTime;
use diesel::prelude::*;
use diesel::result::Error;
use serde::{Deserialize, Serialize};

#[derive(AsChangeset, Identifiable, Insertable, Queryable, Selectable, Deserialize, Serialize)]
#[diesel(table_name = artists)]
pub struct Artists {
    pub id: String,
    pub name: String,
    pub slug: String,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}

#[derive(Debug, Insertable, Deserialize, Serialize)]
#[diesel(table_name = artists)]
pub struct NewArtist {
    pub id: String,
    pub name: String,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}

impl Artists {
    pub fn find(id: String) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let artist = artists::table.filter(artists::id.eq(id)).first(conn)?;
        Ok(artist)
    }

    pub fn create(artist: NewArtist) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = diesel::insert_into(artists::table)
            .values(artist)
            .get_result(conn)?;
        Ok(playlist)
    }

    pub fn find_by_track(track_id: &str) -> Result<Vec<Artists>, Error> {
        let conn = &mut db::connection()?;
        let tracks: Vec<(String, String)> = artists_tracks::table
            .filter(artists_tracks::track_id.eq(track_id))
            .get_results::<(String, String)>(conn)?;

        let artists = tracks
            .into_iter()
            .map(|(artist_id, _)| Artists::find(artist_id).unwrap())
            .collect::<Vec<Artists>>();

        Ok(artists)
    }
}
