use diesel::prelude::*;
use diesel::result::Error;

use crate::helpers::db;

use super::playlists::{PlaylistTrack, Playlists};
use super::tracks::Tracks;

use crate::schema::{playlists_tracks, tracks};

#[derive(Identifiable, Selectable, Queryable, Associations, Debug)]
#[diesel(belongs_to(Playlists, foreign_key = playlist_id))]
#[diesel(belongs_to(Tracks, foreign_key = track_id))]
#[diesel(table_name = playlists_tracks)]
#[diesel(primary_key(playlist_id, track_id))]
pub struct PlaylistTracks {
    pub playlist_id: String,
    pub track_id: String,
    pub added_at: String,
}

impl PlaylistTracks {
    pub fn get_tracks(playlist: &Playlists) -> Result<Vec<PlaylistTrack>, Error> {
        let conn = &mut db::connection()?;
        let track_ids = PlaylistTracks::belonging_to(playlist).select(playlists_tracks::track_id);

        let tracks: Vec<Tracks> = tracks::table
            .filter(tracks::id.eq_any(track_ids))
            .load::<Tracks>(conn)?;

        let tracks = tracks
            .into_iter()
            .map(PlaylistTrack::from)
            .collect::<Vec<PlaylistTrack>>();

        Ok(tracks)
    }
}
